<?php
/**
 * A representation of a cache file.
 *
 * PHP version 5
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Steffen Hansen <steffen@klaralvdalens-datakonsult.se>
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */

/**
 * A representation of a cache file.
 *
 * $Horde: framework/Kolab_FreeBusy/lib/Horde/Kolab/FreeBusy/Cache/File.php,v 1.1.2.2 2010/06/15 12:00:33 wrobel Exp $
 *
 * Copyright 2004-2008 Klarälvdalens Datakonsult AB
 *
 * See the enclosed file COPYING for license information (LGPL). If you
 * did not receive this file, see http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Steffen Hansen <steffen@klaralvdalens-datakonsult.se>
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */
class Horde_Kolab_FreeBusy_Cache_File {

    /**
     * The suffix of this cache file.
     *
     * @var string
     */
    var $_suffix = '';

    /**
     * Name of the cache file.
     *
     * @var string
     */
    var $_filename;

    /**
     * Full path to the cache file.
     *
     * @var string
     */
    var $_file;

    /**
     * Cache file version.
     *
     * @var int
     */
    var $_version = 1;

    /**
     * Construct the Horde_Kolab_FreeBusy_Cache_File instance.
     *
     * @param string $cache_dir The path to the cache direcory.
     * @param string $suffix    The suffix of the cache file name.
     */
    function Horde_Kolab_FreeBusy_Cache_File($cache_dir, $suffix = null)
    {
        if (!empty($suffix)) {
            $this->_suffix = $suffix;
        }

        $this->_cache_dir = $cache_dir;
    }

    /**
     * Set the full path to the cache file.
     *
     * @param string $filename  The file name of the cache file.
     *
     * @return NULL
     */
    function setFilename($filename)
    {
        $this->_filename  = $filename;
        $this->_file = $this->_cache_dir . '/' . $this->_filename . '.' . $this->_suffix;
    }

    /**
     * Get the full path to the cache file.
     *
     * @return string The full path to the file.
     */
    function getFile()
    {
        return $this->_file;
    }

    /**
     * Get the path to the cache file.
     *
     * @return string The path to the file.
     */
    function getFilename()
    {
        return $this->_filename;
    }

    /**
     * Clean the cache file contents.
     *
     * @return boolean|PEAR_Error True if successful.
     */
    function purge()
    {
        if (file_exists($this->_file)) {
            $result = @unlink($this->_file);
            if (!$result) {
                return PEAR::raiseError(sprintf("Failed removing file %s",
                                                $this->_file));
            }
        }
        return true;
    }

    /**
     * Store data in the cache file.
     *
     * @param mixed $data A reference to the data object.
     *
     * @return boolean|PEAR_Error True if successful.
     */
    function store(&$data)
    {
        /* Create directories if missing */
        $fbdirname = dirname($this->_file);
        if (!is_dir($fbdirname)) {
            $result = $this->_makeTree($fbdirname);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
        }

        /* Store the cache data */
        $fh = fopen($this->_file, 'w');
        if (!$fh) {
            return PEAR::raiseError(sprintf("Failed creating cache file %s!",
                                            $this->_file));
        }
        fwrite($fh, serialize(array('version' => $this->_version,
                                    'data' => $data)));
        fclose($fh);
        return true;
    }

    /**
     * Load data from the cache file.
     *
     * @return mixed|PEAR_Error The data retrieved from the cache file.
     */
    function &load()
    {
        $file = @file_get_contents($this->_file);
        if ($file === false) {
            return PEAR::raiseError(sprintf("%s failed reading cache file %s!",
                                            get_class($this), $this->_file));
        }
        $cache = @unserialize($file);
        if ($cache === false) {
            return PEAR::raiseError(sprintf("%s failed to unserialize cache data from file %s!",
                                            get_class($this), $this->_file));
        }
        if (!isset($cache['version'])) {
            return PEAR::raiseError(sprintf("Cache file %s lacks version data!",
                                            $this->_file));
        }
        $this->_version = $cache['version'];
        if (!isset($cache['data'])) {
            return PEAR::raiseError(sprintf("Cache file %s lacks data!",
                                            $this->_file));
        }
        if ($cache['version'] != $this->_version) {
            return PEAR::raiseError(sprintf("Cache file %s has version %s while %s is required!",
                                            $this->_file, $cache['version'], $this->_version));
        }
        return $cache['data'];
    }

    /**
     * Generate a tree of directories.
     *
     * @param string $dirname The path to a directory that should exist.
     *
     * @return boolean|PEAR_Error True if successful.
     */
    function _maketree($dirname)
    {
        $base = substr($dirname, 0, strrpos($dirname, '/'));
        $base = str_replace(".", "^", $base);
        if (!empty($base) && !is_dir($base)) {
            $result = $this->_maketree($base);
            if (is_a($result, 'PEAR_Error')) {
                return $result;
            }
        }
        if (!file_exists($dirname)) {
            $result = @mkdir($dirname, 0755);
            if (!$result) {
                return PEAR::raiseError(sprintf("Error creating directory %s", $dirname));
            }
        }
        return true;
    }
}
