<?php
/**
 * A cache file for complete free/busy information.
 *
 * PHP version 5
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Steffen Hansen <steffen@klaralvdalens-datakonsult.se>
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */

/**
 * A cache file for complete free/busy information.
 *
 * $Horde: framework/Kolab_FreeBusy/lib/Horde/Kolab/FreeBusy/Cache/File/vcal.php,v 1.1.2.3 2010/06/15 12:00:34 wrobel Exp $
 *
 * Copyright 2004-2008 Klarälvdalens Datakonsult AB
 *
 * See the enclosed file COPYING for license information (LGPL). If you
 * did not receive this file, see http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Steffen Hansen <steffen@klaralvdalens-datakonsult.se>
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */
class Horde_Kolab_FreeBusy_Cache_File_vcal extends Horde_Kolab_FreeBusy_Cache_File {

    /**
     * The suffix of this cache file.
     *
     * @var string
     */
    var $_suffix = 'vc';

    /**
     * Cache file version.
     *
     * @var int
     */
    var $_version = 3;

    /**
     * Cached data.
     *
     * @var array
     */
    var $_data;

    /**
     * Minimum age before a cache file can be considered to be expired.
     *
     * @var int
     */
    var $_min_age;

    /**
     * Maximum age after which a cache file is always considered to be expired.
     *
     * @var int
     */
    var $_max_age;

    /**
     * Construct the Horde_Kolab_FreeBusy_Cache_File_vcal instance.
     *
     * @param string  $cache_dir The path to the cache direcory.
     * @param boolean $extended  Does the cache hold extended data?
     * @param int     $min_age   Minimum age in seconds before a cache file can
     *                           be considered to be expired.
     * @param int     $max_age   Maximum age in seconds after which a cache file
     *                           is always considered to be expired.
     */
    function Horde_Kolab_FreeBusy_Cache_File_vcal(
        $cache_dir, $extended, $min_age = 300, $max_age = 259200
    ) {
        $extension = empty($extended) ? 'vc' : 'xvc';
        $this->_min_age = $min_age;
        $this->_max_age = $max_age;
        parent::Horde_Kolab_FreeBusy_Cache_File($cache_dir, $extension);
    }

    /**
     * Store free/busy infomation in the cache file.
     *
     * @param Horde_iCalendar $vcal   A reference to the data object.
     * @param array           $mtimes A list of modification times for the 
     *                                partial free/busy cache times.
     *
     * @return boolean|PEAR_Error True if successful.
     */
    function storeVcal($vcal, $mtimes, $signature)
    {
        $data = array(
            'vcal'      => $vcal,
            'mtimes'    => $mtimes,
            'signature' => $signature
        );
        return $this->store($data);
    }

    /**
     * Load the free/busy information from the cache.
     *
     * @return Horde_iCalendar|PEAR_Error The retrieved free/busy information.
     */
    function &loadVcal()
    {
        if ($this->_data) {
            return $this->_data;
        }

        $result = $this->load();
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        $this->_data = $result['vcal'];

        return $this->_data;
    }

    /**
     * Check if the cached free/busy expired.
     *
     * @param array  $files     A list of partial free/busy cache files.
     * @param string $signature The current signature.
     * @param string $signature The current signature.
     *
     * @return boolean|PEAR_Error True if the cache expired.
     */
    function expired(
        Horde_Kolab_FreeBusy_Export_Freebusy_Combined $combined,
        $extended
    ) {
        $result = $this->load();
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        /* Check the cache version */
        if ($this->_version !== 3) {
            return true;
        }

        $this->_data = $result['vcal'];

        /* Age of the cache file. */
        $components = $this->_data->getComponents();
        foreach ($components as $component) {
            if ($component->getType() == 'vFreebusy') {
                $attr = $component->getAttribute('DTSTAMP');
                if (!empty($attr) && !is_a($attr, 'PEAR_Error')) {
                    $dtstamp = (int)$attr;
                }
            }
        }

        if (time() - $dtstamp < $this->_min_age) {
            return false;
        }

        if ($combined->hasRemoteServers()) {
            return true;
        }

        /* Signature changed? */
        if ($result['signature'] !== $combined->getSignature($extended)) {
            return true;
        }

        /* Check the file mtimes */
        $files = $combined->getFiles();
        foreach ($files as $file) {
            if (filemtime($result['mtimes'][$file][0]) != $result['mtimes'][$file][1]) {
                return true;
            }
        }

        if (time() - $dtstamp > $this->_max_age) {
            return true;
        }

        return false;
    }
}
