<?php
/**
 * Test fetching the free/busy information from the cache.
 *
 * PHP version 5
 *
 * @category   Kolab
 * @package    Kolab_FreeBusy
 * @subpackage UnitTests
 * @author     Gunnar Wrobel <wrobel@pardus.de>
 * @license    http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link       http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */

/**
 * Prepare the test setup.
 */
require_once dirname(__FILE__) . '/../Autoload.php';

require_once 'Horde/iCalendar.php';
require_once 'Horde/Kolab/FreeBusy/Access.php';
require_once 'Horde/Kolab/FreeBusy/Cache.php';
require_once 'Horde/Kolab/Server/Object.php';

/**
 * Test combining partial free/busy scripts into a single vCalendar.
 *
 * Copyright 2010 Klarälvdalens Datakonsult AB
 *
 * See the enclosed file COPYING for license information (LGPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
 *
 * @category   Kolab
 * @package    Kolab_FreeBusy
 * @subpackage UnitTests
 * @author     Gunnar Wrobel <wrobel@pardus.de>
 * @license    http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link       http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */
class Horde_Kolab_FreeBusy_Class_CacheTest
extends PHPUnit_Framework_TestCase
{
    public function setUp()
    {
        global $conf;

        $this->cache_dir = Horde::getTempDir() . '/Kolab_FreeBusy_Test_' . md5(uniqid(mt_rand(), true));

        $this->access = $this->getMock('Horde_Kolab_FreeBusy_Access');

        $this->access->user_object = $this->getMock('Horde_Kolab_Server_Object_user', array('getGroupAddresses'), array(), '', false, false);
        $this->access->user = 'anonymous@example.org';
        $this->access->user_object->expects($this->any())
            ->method('getGroupAddresses')
            ->will($this->returnValue(array()));

        $this->access->owner_object = $this->getMock('Horde_Kolab_Server_Object', array('get'), array(), '', false, false);
        $this->access->owner_object->expects($this->any())
            ->method('get')
            ->with(KOLAB_ATTR_CN)
            ->will($this->returnValue('Mr. Test'));
        $this->access->owner = 'test@example.org';


        $this->cache = new Horde_Kolab_FreeBusy_Cache(
            $this->cache_dir
        );

        $conf = array();
        $conf['log']['enabled'] = true;
        $conf['log']['type'] = 'null';
        $conf['log']['name'] = 'mock';
        $conf['log']['ident'] = 'mock';
        $conf['log']['params'] = array();
        $conf['log']['priority'] = PEAR_LOG_DEBUG;
        $log = Horde::getLogger();
        $this->observer = new Log_Observer_Mock(PEAR_LOG_DEBUG);
        $log->attach($this->observer);
    }

    public function tearDown()
    {
        foreach (new RecursiveIteratorIterator(new RecursiveDirectoryIterator($this->cache_dir), RecursiveIteratorIterator::CHILD_FIRST) as $file) {
            if ($file->isFile()) {
                unlink($file->getPathname());
            }
        }
        foreach (new RecursiveIteratorIterator(new RecursiveDirectoryIterator($this->cache_dir), RecursiveIteratorIterator::CHILD_FIRST) as $file) {
            if ($file->isDir()) {
                rmdir($file->getPathname());
            }
        }
        rmdir($this->cache_dir);
    }

    public function testMethodLoadReturnsEmptyVCalendar()
    {
        $vfb = $this->cache->load($this->access, true)->findComponent('vfreebusy');
        $this->assertEquals(
            'This is a dummy vfreebusy that indicates an empty calendar',
            $vfb->getAttribute('COMMENT')
        );
    }

    public function testMethodLoadReturnsCachedVCalendarWithOldCommonName()
    {
        $vfb = $this->cache->load($this->access, true)->findComponent('vfreebusy');
        $this->assertEquals(
            array(array('cn' => 'Mr. Test')),
            $vfb->getAttribute('ORGANIZER', true)
        );

        $this->access->owner_object = new Owner_Stub();

        $vfb = $this->cache->load($this->access, true)->findComponent('vfreebusy');
        $this->assertEquals(
            array(array('cn' => 'Mr. Test')),
            $vfb->getAttribute('ORGANIZER', true)
        );
    }

    public function testMethodLoadReturnsCachedVCalendarWithNewCommonNameIfCacheExpired()
    {
        $vfb = $this->cache->load($this->access, true)->findComponent('vfreebusy');
        $this->assertEquals(
            array(array('cn' => 'Mr. Test')),
            $vfb->getAttribute('ORGANIZER', true)
        );

        $this->access->owner_object = new Owner_Stub();
        unlink($this->cache_dir . '/example^org/anonymous/example^org/test.xvc');

        $vfb = $this->cache->load($this->access, true)->findComponent('vfreebusy');
        $this->assertEquals(
            array(array('cn' => 'Mr. New')),
            $vfb->getAttribute('ORGANIZER', true)
        );
    }

    public function testMethodLoadReturnsCachedVCalendarWithNewCommonNameIfMinAgeIsBelowZero()
    {
        global $conf;

        $conf['fb']['vcal_cache']['min_age'] = -1;

        $vfb = $this->cache->load($this->access, true)->findComponent('vfreebusy');
        $this->assertEquals(
            array(array('cn' => 'Mr. Test')),
            $vfb->getAttribute('ORGANIZER', true)
        );

        $this->access->owner_object = new Owner_Stub();

        $vfb = $this->cache->load($this->access, true)->findComponent('vfreebusy');
        $this->assertEquals(
            array(array('cn' => 'Mr. New')),
            $vfb->getAttribute('ORGANIZER', true)
        );
    }

    public function testMethodLoadReturnsSameCachedVCalendarIfMinAgeIsBelowZero()
    {
        global $conf;

        $conf['fb']['vcal_cache']['min_age'] = -1;

        $vfb = $this->cache->load($this->access, true)->findComponent('vfreebusy');
        $this->assertEquals(
            array(array('cn' => 'Mr. Test')),
            $vfb->getAttribute('ORGANIZER', true)
        );
        $vfb = $this->cache->load($this->access, true)->findComponent('vfreebusy');
        $this->assertEquals(
            array(array('cn' => 'Mr. Test')),
            $vfb->getAttribute('ORGANIZER', true)
        );
    }
}